/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package an;

import java.io.File;

import org.opengion.fukurou.business.BizLogic_ENTRY;
import org.opengion.fukurou.security.HybsCryptography;
import org.opengion.fukurou.util.FileUtil;
import org.opengion.hayabusa.common.HybsSystem;

/**
 * ユーザー登録 (OP1010) の業務ロジックです。
 *
 * @og.rev 2010/06/18 新規作成
 * 
 * @version 5.0
 * @author Hiroki Nakamura
 * @since JDK1.6,
 */

public class OP1010A extends BizLogic_ENTRY {

	/**
	 * チェックロジックを定義します。
	 * 
	 * @return 処理が正常終了したか
	 */
	@Override
	protected boolean check() {
		// ANE0015:指定されたIDは既に登録されています。ID={0}
		String chk = "select count(*) CNT from AN04 where USERID='{@USERID}'";
		sql( chk );
		if( vari( "CNT" ) > 0 ) {
			error( 2, "ANE0015", var( "USERID" ) );
			return false;
		}

		// ANE0018:指定されたメールアドレスは既に登録されています。メールアドレス={0}
		chk = "select count(*) CNT from AN04 where MAILAD='{@MAILAD}'";
		sql( chk );
		if( vari( "CNT" ) > 0 ) {
			error( 2, "ANE0018", var( "MAILAD" ) );
			return false;
		}

		// ANE0019:パスワードが一致していません。
		if( !var( "PASSWD").equals( var( "PASSWD2" ) ) ) {
			error( 2, "ANE0019" );
		}

		// ANE0020:メールアドレスが一致していません。
		if( !var( "MAILAD" ).equals( var( "MAILAD2" ) ) ) {
			error( 2, "ANE0020" );
		}

		// ANE0021:パスワードは、6文字以上で登録して下さい。
		if( var( "PASSWD" ).length() < 6 ) {
			error( 2, "ANE0021" );
		}

		// ANE0022:メールアドレスが不正です。
		if( var( "MAILAD" ).indexOf( '@' ) < 0 || var( "MAILAD" ).indexOf( '.' ) < 0 ) {
			error( 2, "ANE0022" );
		}

		// ANE0025:画像以外のファイルが指定されています。ファイル名={0}
		if( var( "IMGUSER" ) != null && var( "IMGUSER" ).length() > 0 ) {
			if( !Utils.checkImageName( var( "IMGUSER" ) ) ) {
				error( 2, "ANE0025", var( "IMGUSER" ) );
			}
		}

		return true;
	}

	/**
	 * メイン処理を定義します。
	 * 
	 * @return 処理が正常終了したか
	 */
	@Override
	protected boolean logic() {
		String sql = "insert into AN04 ("
			+ "USERID,PASSWD,ROLES,NAME_JA,MAILAD,UPROFILE,IMGUSER"
			+ ",FGJ,DYSET,PGSET,PGPSET,USRSET,DYUPD,PGUPD,PGPUPD,USRUPD"
			+ ") values ("
			+ "'{@USERID}','{@PASSWD}','USER','{@NAME_JA}','{@MAILAD}','{@UPROFILE}','{@IMGUSER}' "
			+ ",'0','{@CON.YMDH}','{@CON.PGID}','{@CON.PGPID}','{@CON.USERID}','{@CON.YMDH}','{@CON.PGID}','{@CON.PGPID}','{@CON.USERID}'"
			+ ")";
		sql( sql );

		if( var( "IMGUSER" ) != null && var( "IMGUSER" ).length() > 0 ) {
			String imgOrigPath = HybsSystem.sys( "REAL_PATH" ) + HybsSystem.sys( "FILE_URL" )
									+ var( "TEMPDIR" ) + File.separator;

			String imgNewPath = HybsSystem.sys( "REAL_PATH" ) + "userimages" + File.separator
									+ var( "USERID" ) + File.separator;

			new File( imgNewPath ).mkdirs();
			FileUtil.copy( imgOrigPath + var( "IMGUSER" ), imgNewPath + var( "IMGUSER" ) );
			FileUtil.deleteFiles( new File ( imgOrigPath ) );
		}

		rtn( ( new HybsCryptography() ).encrypt( var( "USERID" ) ) );

		return true;
	}
}

